import config, random, datetime

class EventManager:
    def __call__(self):
        return self
    
    def __init__(self):
        self.subscribers = []

    def subscribe(self,subscriber):
        self.subscribers.append(subscriber)

    def unsubscribe(self,subscriber):
        if self.subscribers.count(subscriber) > 0:
            self.subscribers.remove(subscriber)

    def notify(self,sender,verb="changed",*args,**kwargs):
        for subscriber in self.subscribers:
            method = "on_%s_%s" % (sender.__class__.__name__.lower(),verb)
            if hasattr(subscriber,method):
                getattr(subscriber,method)(sender,*args,**kwargs)

EventManager = EventManager()

class Agent(object):
    def __init__(self):
        self.id = random.randrange(10000)
        self.subscribers = []
        self.em = EventManager()
        self.em.subscribe(self)

    def notify(self,verb="changed",*args,**kwargs):
        self.em.notify(self,verb,*args,**kwargs)

    def debug(self,message):
        if config.debug:
            print datetime.datetime.now().isoformat(),
            print self.__class__.__name__ + "(%d): %s" % (self.id,message)

class Engine(Agent):
    def __init__(self, width, height):
    	Agent.__init__(self)
        self.width = width
        self.height = height
        self.updated = True

    def clear(self):
        pass

    def stop(self):
        pass

    def update(self):
        pass

class Creature(Agent):

    def __init__(self,geno = None,mask=0):
        Agent.__init__(self)
        random.seed(datetime.datetime.now().microsecond)
        if geno == None:
            geno = random.getrandbits(4)
            self.debug(geno)
        self.geno = geno
    #    self.hash = int(random.getrandbits(31))

    #def __hash__(self):
    #    return self.hash

    def get_pheno(self):
        return (self.geno >> 2 & 3, self.geno & 3)

    def set_pheno(self,pheno):
        self.geno = pheno[1] << 2 + pheno[0]

    def create_random_mask(self, zero=True):
        mask = 3 * random.getrandbits(1) + 12 * random.getrandbits(1)
        if mask == 0L and not zero:
            mask += 3 << (2 * random.randrange(0,2))
        return mask

    pheno = property(get_pheno,set_pheno)


